#!/usr/bin/env python

import cv2
import numpy as np
import math

# Wir definieren hier einen HSV-Farbbereich mit Startwerten
targetColorLow  = np.array([70,205,60])
targetColorHigh = np.array([130,255,255])

CalibWindowName = 'Color Calibration'

def nothing(*arg):
  pass

def createCalibUi():

  cv2.namedWindow(CalibWindowName)

  # Wir erstellen Schieberegler für die oberen und unteren Grenzwerte
  cv2.createTrackbar('lowHue',  CalibWindowName, targetColorLow[0],  255, nothing)
  cv2.createTrackbar('highHue', CalibWindowName, targetColorHigh[0], 255, nothing)

  cv2.createTrackbar('lowSat',  CalibWindowName, targetColorLow[1],  255, nothing)
  cv2.createTrackbar('highSat', CalibWindowName, targetColorHigh[1], 255, nothing)

  cv2.createTrackbar('lowVal',  CalibWindowName, targetColorLow[2],  255, nothing)
  cv2.createTrackbar('highVal', CalibWindowName, targetColorHigh[2], 255, nothing)

  cv2.waitKey(1)

def updateCalibFromUi():

  # Wir fragen die aktuellen Werte ab und weisen sie unseren HSV-Variablen zu
  lowHue =  cv2.getTrackbarPos('lowHue',  CalibWindowName)
  lowSat =  cv2.getTrackbarPos('lowSat',  CalibWindowName)
  lowVal =  cv2.getTrackbarPos('lowVal',  CalibWindowName)
  highHue = cv2.getTrackbarPos('highHue', CalibWindowName)
  highSat = cv2.getTrackbarPos('highSat', CalibWindowName)
  highVal = cv2.getTrackbarPos('highVal', CalibWindowName)

  global targetColorLow
  global targetColorHigh

  targetColorLow  = np.array([lowHue, lowSat, lowVal])
  targetColorHigh = np.array([highHue,highSat,highVal])

  print(lowHue, lowSat, lowVal, highHue, highSat, highVal)


def findEnclosingCircle(image) :
  # Der Gaußsche Weichzeichner zeichnet das Bild weich
  # Wir entfernen dadurch harte Kanten
  blur = cv2.GaussianBlur(image, (7, 7), 0)

  # HSV (Hue, Saturation, Value)
  hsv = cv2.cvtColor(blur, cv2.COLOR_BGR2HSV)

  mask = cv2.inRange(hsv, targetColorLow, targetColorHigh)

  kernal = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (7, 7))
  mask = cv2.morphologyEx(mask, cv2.MORPH_CLOSE, kernal)
  mask = cv2.morphologyEx(mask, cv2.MORPH_OPEN, kernal)

  bmask = cv2.GaussianBlur(mask, (5,5),0)

  contours, hierarchy = cv2.findContours(bmask, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)

  if contours:
    cnt = contours[0]
    (x,y),radius = cv2.minEnclosingCircle(cnt)

    im2 = image.copy()
    cv2.rectangle(im2,(0,0),(320,240),(0,0,0), -1)
    cv2.circle(im2,(int(x),int(y)),int(radius),(255,255,255), -1)
    
    im2=cv2.bitwise_and(image, im2)

    cv2.addWeighted(image, 0.2, im2, 0.8, 0, image)

    cv2.line(image,(160,0),(160,240),(0,255,0))
    cv2.line(image,(0,int(y)),(320,int(y)),(0,255,0))
    cv2.circle(image,(int(x),int(y)),int(radius),(0,255,0), 2)


    return (x,y, radius)


def putTargetInfo(image, target):

  font                   = cv2.FONT_HERSHEY_SIMPLEX
  fontScale              = 0.5
  fontColor              = (0,255,0)
  lineType               = 2

  if target:
    cv2.putText(image, ('alpha: %s grad' % target['alpha']), (170,20), font, fontScale, fontColor, lineType)
    cv2.putText(image, ('distance: %s cm' % target['distance']), (170,40), font, fontScale, fontColor, lineType)
  else:
    cv2.putText(image, ('not found'), (170,20), font, fontScale, fontColor, lineType)


def findTarget(image):

  foundCircle = findEnclosingCircle(image)

  if foundCircle:

    (xd,yd,rd) = foundCircle
    
    factor = int(25.0 /(2*rd))
    distance = double(factor * 100.0)
    perception = distance/320.0

    width = int(rd * 2 * perception)
    offset = int((xd-160) * perception)
    
    rad2grad = 57.296
    alpha = int(math.atan(offset*1.0/distance)*rad2grad)

    target = {
     'distance' : distance,
     'width'    : width,
     'offset'   : offset,
     'alpha'    : alpha
    }

    putTargetInfo (image, target)
    return target

#-------------------------

global scanInput

def scan_init() :
  global scanInput
  scanInput = cv2.VideoCapture(0)

  scanInput.set( cv2.cv.CV_CAP_PROP_FRAME_WIDTH,  320 );
  scanInput.set( cv2.cv.CV_CAP_PROP_FRAME_HEIGHT, 240 );

  createCalibUi()

def handleUi():
  for i in range(20):
    cv2.waitKey(1) & 0xFF

def scan_cleanup():
  cv2.destroyAllWindows()

def scan():

  global scanInput
  ok, im = scanInput.read()

  if ok:
    image = im.copy()
    updateCalibFromUi()
    target = findTarget(image)

    cv2.imshow("target", image)
    cv2.waitKey(1)

    if target:
      print("scan: target -> ", target)
    else:
      print("scan: no target found")

    return target

#-------------------------

if __name__ == "__main__":

  scan_init()

  while True:
    scan()
    handleUi()

  scan_cleanup()


